#!/usr/bin/env python3

"""
This is the entrypoint for the `iccicd` application.

CLI arguments and handlers for individual modules are
registered here and defined in the respective `cli` submodules.

`launch_common()` is common code run before all handlers.
"""

import argparse
import logging

from iccore import runtime, logging_utils

import iccicd.packaging.cli
import iccicd.ci.cli
import iccicd.repo.cli

logger = logging.getLogger(__name__)


def launch_common(args) -> None:
    """
    Run this code before all CLI handlers. It sets up a global
    runtime context to store runtime configuration, such as the
    'dry run' flag and sets up logging.
    """
    runtime.ctx.set_is_dry_run(args.dry_run)
    logging_utils.setup_default_logger()


def main_cli():
    """
    This method is associated with the `iccicd` application in the
    project's pyproject.toml, i.e. it will be run when `iccicd` is
    called in a terminal.
    """

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--dry_run",
        type=int,
        default=0,
        help="Dry run script - 0 can modify, 1 can read, 2 no modify - no read",
    )
    subparsers = parser.add_subparsers(required=True)

    iccicd.packaging.cli.register(subparsers)
    iccicd.repo.cli.register(subparsers)
    iccicd.ci.cli.register(subparsers)

    args = parser.parse_args()
    launch_common(args)
    args.func(args)


if __name__ == "__main__":
    main_cli()



"""
CLI handling for the 'repo' module.
"""

import logging
from pathlib import Path
import os

from iccore.project import version

from iccicd import repo
from iccicd.repo import python_repo


logger = logging.getLogger(__name__)


def set_version(args) -> None:
    """
    Set the project's version via its configuration files. At the moment
    only Python is supported.
    """

    logger.info("Setting version number")

    python_repo.set_version(
        repo.Repo(path=args.repo_dir, version=version.parse(args.version))
    )

    logger.info("Finished setting version number")


def increment_tag_cli(args) -> None:
    """
    Increment the repo's git tag
    """

    logger.info("Incrementing tag")

    repo.increment_tag(
        args.repo_dir,
        args.field,
        args.url,
        repo.Secret(data=args.token),
        args.user_name,
        args.user_email,
    )

    logger.info("Finished incrementing tag")


def register(subparsers):
    """
    Register the module's CLI handlers
    """

    set_version_parser = subparsers.add_parser("set_version")
    set_version_parser.add_argument(
        "--repo_dir",
        type=Path,
        default=Path(os.getcwd()),
        help="Path to the repo to set the version",
    )

    set_version_parser.add_argument(
        "version",
        type=str,
        help="The version to set",
    )
    set_version_parser.set_defaults(func=set_version)

    increment_tag_parser = subparsers.add_parser("increment_tag")
    increment_tag_parser.add_argument(
        "--repo_dir",
        type=Path,
        default=Path(os.getcwd()),
        help="Path to the repo to increment the tag",
    )

    increment_tag_parser.add_argument(
        "--field",
        type=str,
        default="patch",
        help="The tag field to increment: 'major', 'minor' or 'patch'",
    )
    increment_tag_parser.add_argument(
        "--user_name", type=str, default="", help="Name of the CI user"
    )
    increment_tag_parser.add_argument(
        "--user_email", type=str, default="", help="Email of the CI user"
    )
    increment_tag_parser.add_argument(
        "--url", type=str, default="", help="Url for the repo remote"
    )
    increment_tag_parser.add_argument(
        "--token", type=str, default="", help="Oath access token for the repo"
    )
    increment_tag_parser.set_defaults(func=increment_tag_cli)



from pydantic import BaseModel


class MergeRequest(BaseModel, frozen=True):

    source_branch: str = ""
    target_branch: str = ""
    title: str = ""
    description: str = ""
    remove_source_branch: bool = False
    squash_on_merge: bool = False



import logging
from pathlib import Path
import os
import shutil

from iccore.system import process
from iccore import filesystem as fs
from iccore.version_control import (
    GitlabClient,
    GitlabInstance,
    GitRepo,
    GitUser,
    GitlabToken,
)
from iccore.version_control.gitlab_client import get_latest_release

logger = logging.getLogger(__name__)


def gitlab_ci_push(
    user_name: str,
    user_email: str,
    repo_url: str,
    token: str,
    message: str,
    repo_dir: Path = Path(os.getcwd()),
):

    user = GitUser(name=user_name, email=user_email)
    repo = GitRepo(user=user, path=repo_dir)

    gitlab = GitlabClient(
        instance=GitlabInstance(url=repo_url),
        token=GitlabToken(value=token),
        user=user,
        local_repo=repo,
    )
    gitlab.push_change(message)


def sync_external_archive(
    repo_dir: Path,
    project_id: int,
    asset_name: str,
    archive_name: str,
    sync_script: Path,
    source_token: str = "",
    user_name: str = "",
    user_email: str = "",
    url: str = "",
    target_token: str = "",
    target_repo_url: str = "",
):

    # Download package
    logger.info("Fetching remote asset from %s for project %s", url, project_id)
    work_dir = repo_dir / "_temp"
    os.makedirs(work_dir)
    version = get_latest_release(
        project_id, url, source_token, "PRIVATE-TOKEN", asset_name, work_dir
    )

    logger.info("Extracting archive from %s", work_dir / asset_name)
    fs.unpack_archive(work_dir / asset_name, work_dir)
    (work_dir / asset_name).unlink()

    # Run sync script
    logger.info("Running sync with %s", repo_dir / sync_script)
    process.run(f"{repo_dir/sync_script} {work_dir}")

    # Clean work working directory
    shutil.rmtree(work_dir)

    # Commit and push change
    logger.info("Pushing change")
    gitlab_ci_push(
        user_name,
        user_email,
        url,
        target_token,
        f"External repo sync: v{version}",
        repo_dir,
    )

    logger.info("Finished external package sync")



from pathlib import Path
import logging

from iccore.project import version, Version

from iccicd.repo import Repo
from iccicd.repo.python_repo import sphinx, pyproject


logger = logging.getLogger(__name__)


def get_version(
    path: Path, pyproject_filename: str = pyproject._DEFAULT_FILENAME
) -> Version:
    return pyproject.get_version(path, pyproject_filename)


def set_version(
    repo: Repo,
    ver: Version | None = None,
    pyproject_filename: str = pyproject._DEFAULT_FILENAME,
    sphinx_filename: Path = sphinx._DEFAULT_CONF,
):
    selected_version = ver if ver else repo.version

    logger.info("Setting project version to %s", selected_version.as_string())

    pyproject.set_version(repo.path, selected_version, pyproject_filename)
    sphinx.set_version(repo.path, selected_version, Path(sphinx_filename))


def increment_version(
    repo: Repo,
    field: str,
    pyproject_filename: str = pyproject._DEFAULT_FILENAME,
    sphinx_filename: Path = sphinx._DEFAULT_CONF,
):
    logger.info(
        "Incrementing project %s version from %s", field, repo.version.as_string()
    )
    set_version(
        repo,
        version.increment(repo.version, field),
        pyproject_filename,
        sphinx_filename,
    )



from pathlib import Path

from pydantic import BaseModel

from iccore.project import Version

from iccore.version_control import git, GitRemote, GitUser


class Secret(BaseModel, frozen=True):
    data: str


class Repo(BaseModel, frozen=True):

    path: Path
    version: Version


def increment_tag(
    repo_path: Path,
    field: str,
    url: str = "",
    token: Secret | None = None,
    user_name: str = "",
    user_email: str = "",
) -> None:

    remote: str | None = None
    if url and token:
        remote = "oath_remote"
        url_prefix = f"https://oauth2:{token.data}"
        git.add_remote(repo_path, GitRemote(name=remote, url=f"{url_prefix}@{url}"))

    if user_name and user_email:
        git.set_user(repo_path, GitUser(name=user_name, email=user_email))

    git.increment_tag(repo_path, field=field, remote=remote)



"""
CLI handlers for the `packaging` module
"""

import os
from pathlib import Path
import logging

from iccicd.packaging.python_package import deploy


logger = logging.getLogger(__name__)


def deploy_cli(args):

    logger.info("Doing deployment")

    deploy(args.repo_dir, args.token, args.use_test_repo)

    logger.info("Finished deployment")


def register(subparsers):

    parser = subparsers.add_parser("deploy")
    parser.add_argument(
        "--repo_dir",
        type=Path,
        default=Path(os.getcwd()),
        help="Path to the repo to be deployed",
    )

    parser.add_argument(
        "--token",
        type=str,
        default="",
        help="Authentication token for the target repo",
    )

    parser.add_argument(
        "--use_test_repo",
        type=bool,
        default=False,
        help="If there is an available test repo use it.",
    )
    parser.set_defaults(func=deploy_cli)
